Package: futurize
Version: 1.0.0
Title: Parallelize Common Functions via One Magic Function
Description: The futurize() function turns sequential map-reduce functions such as base::lapply(), purrr::map(), 'foreach::foreach() %do% { ... }' into concurrent alternatives, providing you with a simple, straightforward path to scalable parallel computing via the 'future' ecosystem <doi:10.32614/RJ-2021-048>. By combining this transpiler function with R's native pipe operator, you have a convenient way for speeding up iterative computations with minimal refactoring, e.g. 'lapply(xs, fcn) |> futurize()', 'purrr::map(xs, fcn) |> futurize()', and 'foreach::foreach(x = xs) %do% { fcn(x) } |> futurize()'. Other map-reduce packages that can be "futurized" are 'BiocParallel', 'plyr', 'crossmap', 'pbapply' packages. There is also support for a growing set of domain-specific packages on CRAN (e.g. 'boot', 'caret', 'DiceKriging', 'ez', 'fgsea', 'fwb', 'gamlss', 'glmmTMB', 'glmnet', 'kernelshap', 'lme4', 'metafor', 'mgcv', 'modelsummary', 'parameters', 'partykit', 'pls', 'pvclust', 'riskRegression', 'rugarch', 'sandwich', 'seriation', 'shapr', 'Sim.DiffProc', 'SimDesign', 'stars', 'strucchange', 'SuperLearner', 'tm', 'TSP', and 'vegan') and on Bioconductor (e.g. 'DESeq2', 'GenomicAlignments', 'GSVA', 'Rsamtools', 'scater', 'scuttle', 'SingleCellExperiment', and 'sva').
Authors@R: c(
   person("Henrik", "Bengtsson", role=c("aut", "cre", "cph"),
          email = "henrikb@braju.com",
          comment = c(ORCID = "0000-0002-7579-5165")))
License: Apache License (>= 2)
URL: https://futurize.futureverse.org,
        https://github.com/futureverse/futurize
BugReports: https://github.com/futureverse/futurize/issues
Depends: R (>= 4.1.0), future (>= 1.69.0)
Imports: utils
Suggests: methods, future.apply (>= 1.20.2), foreach, doFuture (>=
        1.2.1), purrr, furrr, crossmap, plyr, pbapply, BiocParallel,
        boot, survival, caret, randomForest, DESeq2, DiceKriging, ez,
        fgsea, GenomicAlignments, Rsamtools, fwb, gamlss, glmmTMB,
        glmnet, GSVA, kernelshap, lme4, metafor, mgcv, modelsummary,
        parameters (>= 0.29.1), partykit, pls, pvclust, riskRegression,
        rugarch, sandwich, scater, scuttle, seriation, shapr,
        Sim.DiffProc, SimDesign, SingleCellExperiment, stars,
        strucchange, SuperLearner, sva, tm, vegan, tools, commonmark,
        base64enc
VignetteBuilder: futurize
Language: en-US
Encoding: UTF-8
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-06-12 14:06:47 UTC; hb
Author: Henrik Bengtsson [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-7579-5165>)
Maintainer: Henrik Bengtsson <henrikb@braju.com>
Repository: CRAN
Date/Publication: 2026-06-12 15:00:02 UTC
