# Generated by roxygen2: do not edit by hand

S3method(print,transpiled_call)
export(futurize)
export(futurize_options)
export(futurize_supported_functions)
export(futurize_supported_packages)
export(fz)
importFrom(future,plan)
importFrom(utils,capture.output)
importFrom(utils,getS3method)
importFrom(utils,globalVariables)
importFrom(utils,packageName)
importFrom(utils,packageVersion)
