#! /usr/bin/env Rscript
## This runs testme test script inst/testme/test-futurize-SingleCellExperiment.R
## Don't edit - it was autogenerated by inst/testme/deploy.R
futurize:::testme("futurize-SingleCellExperiment")
