#! /usr/bin/env Rscript
## This runs testme test script inst/testme/test-future.apply-options,nested.R
## Don't edit - it was autogenerated by inst/testme/deploy.R
futurize:::testme("future.apply-options,nested")
