<!--
%\VignetteIndexEntry{Parallelize 'foreach' functions}
%\VignetteAuthor{Henrik Bengtsson}
%\VignetteKeyword{R}
%\VignetteKeyword{package}
%\VignetteKeyword{foreach}
%\VignetteKeyword{vignette}
%\VignetteKeyword{futurize}
%\VignetteEngine{futurize::selfonly}
-->

<div class="logos">
<img src="../man/figures/cran-foreach-logo.webp" alt="The CRAN 'foreach' package">
<span>+</span>
<img src="../man/figures/futurize-logo.webp" alt="The 'futurize' hexlogo">
<span>=</span>
<img src="../man/figures/future-logo.webp" alt="The 'future' logo">
</div>

The **futurize** package allows you to easily turn sequential code
into parallel code by piping the sequential code to the `futurize()`
function. Easy!


# TL;DR

```r
library(futurize)
plan(multisession)
library(foreach)

slow_fcn <- function(x) {
  message("x = ", x)
  Sys.sleep(0.1)  # emulate work
  x^2
}

xs <- 1:10
ys <- foreach(x = xs) %do% slow_fcn(x) |> futurize()
```

# Introduction

This vignette demonstrates how to use this approach to parallelize
functions such as `foreach()` and `times()` of the
**[foreach]** package. For example, consider:

```r
library(foreach)
xs <- 1:1000
ys <- foreach(x = xs) %do% slow_fcn(x)
```

This `foreach()` construct is resolved sequentially. We can use the
**futurize** package to tell **foreach** to hand over the
orchestration of parallel tasks to futureverse. All we need to do is
to pass the expression to `futurize()` as in:

```r
library(foreach)

library(futurize)
plan(multisession) ## parallelize on local machine

xs <- 1:1000
ys <- foreach(x = xs) %do% slow_fcn(x) |> futurize()
#> x = 1
#> x = 2
#> x = 3
#> ...
#> x = 10
```

Note how messages produced on parallel workers are relayed as-is back
to the main R session as they complete. Not only messages, but also
warnings and other types of conditions are relayed back as-is.
Likewise, standard output produced by `cat()`, `print()`, `str()`, and
so on is relayed in the same way. This is a unique feature of
Futureverse - other parallel frameworks in R, such as **parallel**,
**foreach** with **doParallel**, and **BiocParallel**, silently drop
standard output, messages, and warnings produced on workers. With
**futurize**, your code behaves the same whether it runs sequentially
or in parallel: nothing is lost in translation.

The built-in `multisession` backend parallelizes on your local
computer and it works on all operating systems. There are [other
parallel backends] to choose from, including alternatives to
parallelize locally as well as distributed across remote machines,
e.g.

```r
plan(future.mirai::mirai_multisession)
```

and

```r
plan(future.batchtools::batchtools_slurm)
```

Here is another example that parallelizes `times()` of the
**foreach** package via the futureverse ecosystem:


```r
library(foreach)
library(futurize)
ys <- times(10) %do% rnorm(3) |> futurize()
```


# Supported Functions

The `futurize()` function supports parallelization of the following
**foreach** functions:

 * `foreach(...) %do% { ... }`
 * `times(...) %do% { ... }` with `seed = TRUE` as the default


# Progress Reporting via progressify

For progress reporting, please see the **[progressify]** package. It is
specially designed to work with the Futureverse ecosystem and provide
progress updates from parallelized computations in a near-live
fashion. See the `vignette("futurize-11-apply", package = "futurize")`
for more details and an example.


[progressify]: https://progressify.futureverse.org/
[foreach]: https://cran.r-project.org/package=foreach
[other parallel backends]: https://www.futureverse.org/backends.html
